/* -*- c++ -*-
Copyright  1993 Bengt Mrtensson and Institute for Dynamical Systems.

This file is part of Gnans.

Gnans is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free
Software Foundation; either version 2, or (at your option) any later
version.

Gnans is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Gnans; see the file COPYING.  If not, write to the 
Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. */

// $Id: UserParameter.h,v 1.4 1994/08/19 18:38:31 bengt Exp $

#ifndef _UserParameter_h
#define _UserParameter_h

class rng;
extern double gnans_NaN();

// Data structure for user visible parameters 
class UserParameter {
  double value;
  const char *name_string;	// The name of the parameter 
  const char *doc_string;	// Documentation string 
  bool store_status;		// Should the variable be stored during run?

public:

  void *widget;
  rng *rng_ptr;

  void init(char *nam, char *doc, bool store, double val)
                                        {
					  value = val;
					  name_string = nam;
					  doc_string = doc;
					  store_status = store;
					}
  void init(char *nam, char *doc, bool store)
  					{ init(nam, doc, store, gnans_NaN()); }
 
  UserParameter() 			{ init(NULL, NULL, false);}
  UserParameter(char *nam, char *doc, bool store) 
                                        { init(nam, doc, store); };
  UserParameter(char *nam, char *doc)	{ init(nam, doc, false); }
  UserParameter(char *nam, char *doc, bool /*store*/, double val)
                                        { init(nam, doc, false, val); }

  void setname(const char *newname)	{ name_string = newname; }
  const char *name() const              { return name_string; }
  const char *documentation() const	{ return doc_string; }
  void store(bool y_or_n)		{ store_status = y_or_n; }
  void toggle_store()			{ store_status = !store_status; }
  bool store_me() const			{ return store_status; }

  operator double() const		{ return value; }
  operator double&()			{ return value; }
  operator double*()			{ return &value; }
  operator int() const			{ return int(value); }
  
  double operator-() const		{ return -double(*this); }
  UserParameter operator=(double val)   { (double&)*this = val; return *this; }
  UserParameter operator+=(double val)	{ (double&)*this += val;
					  return *this; }
  UserParameter operator-=(double val)	{ (double&)*this -= val;
					  return *this; }

  double operator+(double x) const	{ return double(*this) + x; }
  double operator-(double x) const	{ return double(*this) - x; }
  double operator*(double x) const	{ return double(*this) * x; }
  double operator/(double x) const	{ return double(*this) / x; }

  double operator++()			{ value++;
					  return value; }

  double operator+(UserParameter up) const
					{ return double(*this) + up.value; }
  double operator-(UserParameter up) const 
					{ return double(*this) - up.value; }
  double operator*(UserParameter up) const 
					{ return double(*this) * up.value; }
  double operator/(UserParameter up) const	
					{ return double(*this) / up.value; }

  int operator<(UserParameter up) const
					{ return double(*this) < up.value; }
  int operator>(UserParameter up) const
					{ return double(*this) > up.value; }
  int operator<=(UserParameter up) const
					{ return double(*this) <= up.value; }
  int operator>=(UserParameter up) const
					{ return double(*this) >= up.value; }
  int operator==(UserParameter up) const
					{ return double(*this) == up.value; }
  int operator!=(UserParameter up) const
					{ return double(*this) != up.value; }

  int operator<(double x) const		{ return double(*this) < x; }
  int operator>(double x) const		{ return double(*this) > x; }
  int operator<=(double x) const	{ return double(*this) <= x; }
  int operator>=(double x) const	{ return double(*this) >= x; }
  int operator==(double x) const	{ return double(*this) == x; }
  int operator!=(double x) const	{ return double(*this) != x; }

  friend /*static*/ inline int operator<(double x, const UserParameter up);
  friend /*static*/ inline int operator>(double x, const UserParameter up);
  friend /*static*/ inline int operator<=(double x, const UserParameter up);
  friend /*static*/ inline int operator>=(double x, const UserParameter up);
  friend /*static*/ inline int operator==(double x, const UserParameter up);
  friend /*static*/ inline int operator!=(double x, const UserParameter up);

  friend /*static*/ inline double operator+(double a, const UserParameter up);
  friend /*static*/ inline double operator-(double a, const UserParameter up);
  friend /*static*/ inline double operator*(double a, const UserParameter up);
  friend /*static*/ inline double operator/(double a, const UserParameter up);
};

/*static*/ inline double
operator+(double a, const UserParameter up) { return a + double(up); }
/*static*/ inline double
operator-(double a, const UserParameter up) { return a - double(up); }
/*static*/ inline double
operator*(double a, const UserParameter up) { return a * double(up); }
/*static*/ inline double
operator/(double a, const UserParameter up) { return a / double(up); }

/*static*/ inline int 
operator<(double x, const UserParameter up)  { return up > x; }
/*static*/ inline int
operator>(double x, const UserParameter up)  { return up < x; }
/*static*/ inline int
operator<=(double x, const UserParameter up) { return up >= x; }
/*static*/ inline int
operator>=(double x, const UserParameter up) { return up <= x; }
/*static*/ inline int
operator==(double x, const UserParameter up) { return up == x; }
/*static*/ inline int
operator!=(double x, const UserParameter up) { return up != x; }

typedef UserParameter *UserParameterPointer;

#endif
