/* -*- c++ -*-
 Copyright (C) 1992 Bengt Martensson and Institute for Dynamical Systems.

This file is part of Gnans.

Gnans is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free
Software Foundation; either version 2, or (at your option) any later
version.

Gnans is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Gnans; see the file COPYING.  If not, write to the 
Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. */

// $Id: gnanslib.h,v 1.14 1994/11/01 12:50:52 bengt Exp bengt $

// This header contains the externally visible data types and functions in
// the library.

#ifndef _gnanslib_h
#define _gnanslib_h

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

// A few useful functions, in overloaded versions
#if 0
inline static int GNU_CONST
abs(int i) 
{
  return i > 0 ? i : -i;
}
#endif
inline static double GNU_CONST
abs(double x) 
{
  return x > 0 ? x : -x;
}

inline static double GNU_CONST
abs(UserParameter x) 
{
  return abs(double(x));
}

inline static double GNU_CONST
sqr(double x)
{
  return x*x;
}
inline static int GNU_CONST
sqr(int x)
{
  return x*x;
}
inline static double GNU_CONST
sqr(UserParameter x)
{
  return sqr(double(x));
}

inline static double GNU_CONST
cube(double x)
{
  return x*x*x;
}
inline static int GNU_CONST
cube(int x) 
{
  return x*x*x;
}
inline static double GNU_CONST
cube(UserParameter x)
{
  return cube(double(x));
}

inline static double GNU_CONST
heaviside(double x)
{
  return x > 0 ? 1 : 0;
}

inline static double GNU_CONST
min(double x, double y)
{
  return x < y ? x : y;
}

inline static double GNU_CONST
max(double x, double y)
{
  return x > y ? x : y;
}

// Take advantage of the hardware sqrt on the sparc. (All Sparcs except for
// some ancient machines, and they can emulate.)
#if defined(__sparc) && defined(__GNUC__) && !defined(sqrt)

#include <math.h>

inline static double 
sparc_hardware_sqrt(double x)
{
  register double result;
  asm("fsqrtd %1, %0" : "=f" (result) : "f" (x));
  return result;
}
#define sqrt(x) sparc_hardware_sqrt(x)
#endif

/**************** chop.cc ****************/
extern void chop(double &x, int digits, int truncate = 0);

/**************** invPhi.cc ****************/
extern double invPhi(double alpha);

/**************** terminate.c ****************/
extern double terminate(int term, int throw_away_last_point = 0);

#include "normalizer.h"
#include "delay.h"
#include "histo.h"
#include "table.h"

#endif
